using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace ShadingDemo
{
	public class ShadingDemoForm : System.Windows.Forms.Form
	{
		private const int numberRingSegments = 60;
		private const double ringRadius = 1.0;
		private const int numberTubeSegments = 30;
		private const double tubeRadius = 0.6;
				
		private const int numberVertices = numberRingSegments * numberTubeSegments;
				
		private Device device;
		private VertexBuffer myVertices;
		private IndexBuffer myIndices;
		private Material myMaterial;
				
		private Matrix worldMatrix = Matrix.RotationAxis(new Vector3(1.0f, 1.0f, -1.0f), 0.6f);

		private System.ComponentModel.Container components = null;

		public ShadingDemoForm()
		{
			InitializeComponent();

			SetStyle(ControlStyles.AllPaintingInWmPaint | ControlStyles.Opaque | ControlStyles.UserPaint, true);

			PresentParameters presentParams = new PresentParameters();
			presentParams.Windowed = true;
			presentParams.SwapEffect = SwapEffect.Discard;
			presentParams.AutoDepthStencilFormat = DepthFormat.D24X8;
			presentParams.EnableAutoDepthStencil = true;				
				
			device = new Device(0, DeviceType.Hardware, this, CreateFlags.HardwareVertexProcessing, presentParams);
				
			// build a torus
		
			myVertices = new VertexBuffer(typeof(CustomVertex.PositionNormal), numberVertices, device, Usage.WriteOnly, 0, Pool.Managed);
			CustomVertex.PositionNormal[] verts = (CustomVertex.PositionNormal[]) myVertices.Lock(0, 0);
			for(int ringIndex = 0; ringIndex < numberRingSegments; ringIndex++)
			{
				double ringAngle = 2.0*Math.PI*ringIndex/numberRingSegments;				
				for(int tubeIndex = 0; tubeIndex < numberTubeSegments; tubeIndex++)
				{					
					double tubeAngle = 2.0*Math.PI*tubeIndex/numberTubeSegments;
					double radialDistance = ringRadius + tubeRadius*Math.Cos(tubeAngle);
					int index = ringIndex*numberTubeSegments + tubeIndex;
					verts[index].Position
						= new Vector3((float)(radialDistance * Math.Cos(ringAngle)),
						(float)(tubeRadius * Math.Sin(tubeAngle)),
						(float)(radialDistance * Math.Sin(ringAngle))
						);
					verts[index].Normal
						= new Vector3((float)(Math.Cos(tubeAngle)*Math.Cos(ringAngle)),
						(float)(Math.Sin(tubeAngle)),
						(float)(Math.Cos(tubeAngle)*Math.Sin(ringAngle))
						);
				}
			}
			myVertices.Unlock();

			myIndices = new IndexBuffer(typeof(short), 6*numberRingSegments*numberTubeSegments, device, Usage.WriteOnly, Pool.Managed);
			short[] idx = (short[]) myIndices.Lock(0, 0);
			for(int ringIndex = 0; ringIndex < numberRingSegments; ringIndex++)
			{
				for(int tubeIndex = 0; tubeIndex < numberTubeSegments; tubeIndex++)
				{					
					int index = ringIndex*numberTubeSegments + tubeIndex;
					idx[6*index    ] = (short) (index);
					idx[6*index + 1] = (short)((index + numberTubeSegments) % numberVertices);
					idx[6*index + 2] = (short)(ringIndex*numberTubeSegments + (tubeIndex+1)%numberTubeSegments);
					idx[6*index + 3] = (short)(((ringIndex+1)*numberTubeSegments + (tubeIndex+1)%numberTubeSegments) % numberVertices);
					idx[6*index + 4] = idx[6*index + 2];
					idx[6*index + 5] = idx[6*index + 1];
				}
			}
			myIndices.Unlock();

			myMaterial = new Material();
			myMaterial.Ambient = Color.Black;
			myMaterial.Emissive = Color.Black;
			myMaterial.Diffuse = Color.Beige;
			myMaterial.Specular = Color.White;
			myMaterial.SpecularSharpness = 80.0f;
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Vom Windows Form-Designer generierter Code
		/// <summary>
		/// Erforderliche Methode fr die Designeruntersttzung. 
		/// Der Inhalt der Methode darf nicht mit dem Code-Editor gendert werden.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(552, 430);
			this.Name = "Form1";
			this.Text = "Form1";

		}
		#endregion

		[STAThread]
		static void Main() 
		{
			Application.Run(new ShadingDemoForm());
		}

		protected override void OnPaint(PaintEventArgs e)
		{
			base.OnPaint (e);

			device.Clear(ClearFlags.Target | ClearFlags.ZBuffer, Color.Blue, 1.0f, 0);
			device.BeginScene();
				
			device.Transform.View = Matrix.LookAtLH(new Vector3(0.0f, 0.0f, -7.0f), new Vector3(), new Vector3(0.0f, 1.0f, 0.0f));
			device.Transform.Projection = Matrix.PerspectiveFovLH(Geometry.DegreeToRadian(20.0f), ClientSize.Width/(float)ClientSize.Height, 0.1f, 50.0f);

			device.RenderState.CullMode = Cull.None;
			//device.RenderState.ShadeMode = ShadeMode.Flat;
			//device.RenderState.FillMode = FillMode.WireFrame;
			device.RenderState.SpecularEnable = true;
			device.RenderState.NormalizeNormals = true;
			device.RenderState.Lighting = true;
			device.RenderState.Ambient = Color.Black;				
				
			device.Lights[0].Type = LightType.Point;
			device.Lights[0].Position = new Vector3(-3.0f, 3.0f, -4.0f);
			device.Lights[0].Attenuation0 = 1.0f;
			device.Lights[0].Attenuation1 = 0.0f;
			device.Lights[0].Attenuation2 = 0.0f;
			device.Lights[0].Range = 1000.0f;
			device.Lights[0].Ambient = Color.Black;
			device.Lights[0].Diffuse = Color.Yellow;
			device.Lights[0].Specular = Color.White;
			device.Lights[0].Enabled = true;
			device.Lights[0].Update();

			device.Lights[1].Type = LightType.Directional;
			device.Lights[1].Direction = new Vector3(-1.0f, -3.0f, 2.0f);
			device.Lights[1].Ambient = Color.Black;
			device.Lights[1].Diffuse = Color.Blue;
			device.Lights[1].Specular = Color.Gray;
			device.Lights[1].Enabled = true;
			device.Lights[1].Update();

			device.Transform.World = worldMatrix;                            
			device.SetStreamSource(0, myVertices, 0);
			device.Indices = myIndices;
			device.VertexFormat = CustomVertex.PositionNormal.Format;				
			device.Material = myMaterial;			
			device.DrawIndexedPrimitives(PrimitiveType.TriangleList, 0, 0, numberVertices, 0, 2*numberRingSegments*numberTubeSegments);
				
			device.EndScene();
			device.Present();
		}

		protected override void OnKeyDown(KeyEventArgs e)
		{
			if(!e.Handled)
			{
				e.Handled = true;
				switch(e.KeyCode)
				{
					case Keys.Escape:
						Close();
						break;
					case Keys.Up:
						worldMatrix = worldMatrix*Matrix.RotationX( 0.1f);
						Invalidate();
						break;
					case Keys.Down:
						worldMatrix = worldMatrix*Matrix.RotationX(-0.1f);
						Invalidate();
						break;					
					case Keys.Left:
						worldMatrix = worldMatrix*Matrix.RotationY( 0.1f);
						Invalidate();
						break;
					case Keys.Right:
						worldMatrix = worldMatrix*Matrix.RotationY(-0.1f);
						Invalidate();
						break;
					default:
						e.Handled = false;
						break;
				}
			}
			base.OnKeyDown(e);
		}
	}
}

